/*
 * Copyright (C) 1992  Board of Regents of the University of Wisconsin
 * on behalf of the Department of Electrical Engineering and Computer
 * Science, University of Wisconsin-Milwaukee, Milwaukee, WI 53201.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The programs in this directory were developed by software engineering
 * teams as part of the course "Introduction to Software Engineering"
 * under the supervision of Professor G. Davida.
 *
 * Please send all changes, enhancements, and other comments about this
 * software to
 *
 *     		soft-eng@cs.uwm.edu
 *
 *			or
 *		
 *		Software Engineering Coordinator
 *		Computer Science
 *   		Department of EECS
 *		University of Wisconsin - Milwaukee
 *		Milwaukee, WI  53201
 *		414-229-4677
 */
/* $Log:	load_file.c,v $
 * Revision 1.2  92/12/31  23:49:29  jimj
 * New version info
 * 
 * Revision 1.1  1992/09/24  23:51:42  jimj
 * Initial revision
 *
 */
#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Label.h>
#include <sys/types.h>
#include "load_file.h"

#define MAXPACKETS 5000
#define MAXLINESIZE 5000

extern Widget edit_one_line();
void data_file();
void reload_file();

/*  Read in the file and store
 *  it in a array
 */

char *data_all[MAXPACKETS];
int data_list[MAXPACKETS];
int numpacketslist;
int numpacketsread;

/*  ***********************
 *  Read the data from a file
 *  *************************
 */

load_file()
{
   FILE *fp, *fopen();
   char buf[MAXLINESIZE];
   char name[30];
   char *dummy = "00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00";

   	/*  Get the filename from the screen */
   data_file(name);
#ifdef DEBUG
   printf("Loading file %s\n", name);
#endif

   data_list[0] = 0;

   	/*  Open the file */

   if ((fp = fopen(name, "r")) == NULL) {
		/*  If we can't open the file, then use some default
		 *  packets
		 */
	fprintf(stderr, "Unable to open %s\n", name);
	numpacketslist = 1;  numpacketsread = 0;
	data_all[0] = XtMalloc(strlen(dummy) + 1);
	strcpy(data_all[0], dummy);
	return;
   }
   	/*  Read in the file */
   numpacketsread = 0;
   while ((fgets(buf, MAXLINESIZE, fp)) != NULL && 
			numpacketsread < MAXPACKETS - 1) {
		/*  Get memory to store in */
	data_all[numpacketsread] = XtMalloc(strlen(buf) + 1);
		/*  Remove the newline  */
	buf[strlen(buf) - 1] = '\0';   
		/*  Copy the packet     */
	strcpy(data_all[numpacketsread], buf);
	numpacketsread++;
   }
   	/*  Assign the last place to null  */
   data_all[numpacketsread] = NULL;

   fclose(fp);

   	/*  Mark the packets we want */
   wanted_packets();

#ifdef DEBUG
   printf("File loaded\n");
#endif

} /*  load file  */

/*  ********************* 
 *  reload the file
 *  *********************
 */

void reload_file(w, client_data, call_data)
Widget w;
caddr_t client_data, call_data;
{
   int loop;

   	/*  Remove all the old packets */
   for (loop = 0; loop < numpacketsread; loop++)
     XtFree(data_all[loop]);

     	/*  Call the load routine to load them in  */
   load_file();

   	/*  Update the screen, with the new packets */
   draw_all();
}  /*  reloads file */

/**  This last part puts up a part on the screen where user can enter 
 ** a new filename to get the data from.
 **/

static Widget load = NULL;

/* *********************
 * Prompt for a filename
 * *********************
 */

void filename_button(parent)
Widget parent;
{
   Arg wargs[2];
   Widget noda;

   XtSetArg(wargs[0], XtNlabel, "Packet File->");
   load = XtCreateManagedWidget("load", labelWidgetClass, parent, wargs,1);
   load = edit_one_line("load", parent);

   XtSetArg(wargs[0], XtNlabel, "Re-Load File");
   noda = XtCreateManagedWidget("reload", commandWidgetClass, parent,  wargs,1);
   XtAddCallback(noda, XtNcallback, reload_file, NULL);
}

/*  ************************************
 *  Use the defualt filename if possible
 *  ************************************
 */
void data_file(name)
char *name;
{
	char *got;
	extern char *getlabel();

	 	/*  The file is loaded before the screen is made, so
	 	 *  check the widget pointer, to see it is there 
	 	 */
	if (load == NULL) {
		strcpy(name, SAVENAME);
		return;
	}

		/*  Get the filename from screen, if blank use default */
	got = getlabel(load);
	if (got[0] == 0)
		strcpy(name, SAVENAME);
	else
		strcpy(name, got);
}

