/*
 * Copyright (C) 1992  Board of Regents of the University of Wisconsin
 * on behalf of the Department of Electrical Engineering and Computer
 * Science, University of Wisconsin-Milwaukee, Milwaukee, WI 53201.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The programs in this directory were developed by software engineering
 * teams as part of the course "Introduction to Software Engineering"
 * under the supervision of Professor G. Davida.
 *
 * Please send all changes, enhancements, and other comments about this
 * software to
 *
 *     		soft-eng@cs.uwm.edu
 *
 *			or
 *		
 *		Software Engineering Coordinator
 *		Computer Science
 *   		Department of EECS
 *		University of Wisconsin - Milwaukee
 *		Milwaukee, WI  53201
 *		414-229-4677
 */

#include <sys/time.h>
#include <X11/Xlib.h>
#include "defs.h"
#include "version.h"

extern Display *mydisplay;
extern Window mywindow;
extern GC mygc;
extern void reset_values();

XRectangle tangles[] = {
	40, 40, 250, 160,     /* size graphs */
	350, 40, 120, 55,     /* type graphs */
	350, 130, 120, 90,    /* Netload over time */
	40, 250, 250, 40,     /* load line */
	330, 245, 120, 55,    /* Network utilization load box */
	40, 330, 150, 75,     /* Bytes load box */
	210, 330, 126, 125,   /* Packets load box */
	350, 330, 120, 30,    /* Other box */
};
int ntangles = 8;     /*  How many things are above */

XSegment lines[] = {
	40, 351, 190, 351,
	210, 351, 336, 351,
	210, 400, 336, 400,
};
int nlines = 3;

/*  Draw the all of the boxes, text, and other lines that are
 *  on the screen.  Accept input from the mouse, and call the
 *  routine to sample data, and the routine to print all the
 *  new info that we just sampled.
 */

basicwindow()
{
    int done = 0;
    struct timeval td, start, stop;
    XEvent myevent;

	/*  Initialize all the vars to zero for just the ones that only
 	 *  need to be initialized once 
 	 */
    setupvars();

	/*  Draw the basic window lines, shapes, text on the screen */
    drawwindow();
    XFlush(mydisplay);

    while (!done) {
    	if (XEventsQueued(mydisplay, QueuedAfterReading) == 0) {
       		/*
        	 * Nothing in queue so update the screen
        	 */
        	XFlush(mydisplay);

		/*  Get starting time */
        	gettimeofday(&start, (struct timezone *)0);

		/*  Go and sample some amount of data  */
        	sampledata();

		/*  Get ending time and compute seconds  */
        	gettimeofday(&stop, (struct timezone *)0);
        	td.tv_sec = stop.tv_sec - start.tv_sec;
        	td.tv_usec = stop.tv_usec - start.tv_usec;
        	if (td.tv_usec < 0)
            	 	td.tv_sec--, td.tv_usec += 1000000;

		/*  Go draw all the new info on the screen */
        	drawgraphs(td.tv_sec + (td.tv_usec / 1000000.) );

   	} else {

      		/*  Something was in queue, so processes it */ 
      		XNextEvent(mydisplay, &myevent);

      		switch(myevent.type) {
      		    case Expose:  		/*  was the event an expose? */
	  	         if(myevent.xexpose.count == 0) {
	      	  	     drawwindow();
 	  		 }
 	  		 break;

      		   case ButtonPress:	/*  button press? check quit box */
  	 	        done = checkbutton(myevent.xbutton.x, myevent.xbutton.y);
     		        break;

      		   case MappingNotify:
	  		XRefreshKeyboardMapping( (XMappingEvent*) &myevent);
      			break;
      		}  /* Switch */

	}  /* if statement */
    } /* while loop */
} /* Display it */

/* 
 *  This function just checks to see where the cursor is when the
 *  button was pressed.. These cordinates here are the cordinates
 *  of the quit box.
 */

int checkbutton(x, y)
int x,y;
{
         /*  If press was in quit box return 1 */
    if ( (x >= 10) && (y >=450) && (x <= 50) && (y <= 490) )
       return(1);

         /*  If press was in reset box, reset counter values */
    if ( (x >= 70) && (y >=450) && (x <= 110) && (y <= 490) )
       reset_values();

         /*  If button pressed but not in anyplace important */
    return(0);
} 

/*
 *  Draws all the stationary stuff in the window.  This stuff
 *  Is what gets updated when a screen refresh is done
 */

drawwindow()
{

	    /*  Print the version header on the top of screen */
	Xprint(130, 14, version);

	    /*  Draw the lines and boxes which make up the screen */
	XDrawRectangles(mydisplay, mywindow, mygc, tangles, ntangles);
	XDrawSegments(mydisplay, mywindow, mygc, lines, nlines);

	Xprint(120, 35, "Packet Size Graph");
	Xprint(50, 213, " 0    65 128 256 512  768 1024 1280");
	Xprint(50, 225, " 64  128 256 512 768 1024 1280 1518 >1518");
	Xprint(15, 45, "100%");
	Xprint(15, 61, " 90%");
	Xprint(15, 77, " 80%");
	Xprint(15, 93, " 70%");
	Xprint(15, 109," 60%");
	Xprint(15, 125," 50%");
	Xprint(15, 141," 40%");
	Xprint(15, 157," 30%");
	Xprint(15, 173," 20%");
	Xprint(15, 189," 10%");
	Xprint(15, 205,"  0%");
	
	Xprint(360, 35, "Packet Type Graph");
	Xprint(310, 70, "  Arp"); 
	Xprint(310, 85, "   IP");
	Xprint(460, 108, "100%");
	Xprint(345, 108, "0%"); 
	
	Xprint(365, 127, "Load Over Time");
	Xprint(325, 135, "100%");
	Xprint(325, 180, " 50%");
	Xprint(325, 220, "  0%");
	
	Xprint(105, 245, "Network load (x10)" );
	Xprint(41, 303, "0   1   2   3   4   5   6   7   8   9   10%");
	
	Xprint(335, 240, "Network Utilization");
	Xprint(335, 260, " Now:      %");
	Xprint(335, 275, " Ave:      %");
	Xprint(335, 290, "Peak:      %");
	
	Xprint(100, 325, "Bytes");
	Xprint(50, 345, "Total:            bytes");
	Xprint(50, 365, "  Now:            b/sec"); 
	Xprint(50, 380, "  Ave:            b/sec");
	Xprint(50, 395, " Peak:            b/sec");
	
	Xprint(260, 325, "Packets");
	Xprint(218, 345, "Total:        paks");
	Xprint(218, 365, "  Now:        pak/s");
	Xprint(218, 380, "  Ave:        pak/s");
	Xprint(218, 395, " Peak:        pak/s");
	Xprint(218, 415, "  Now:        b/pak");
	Xprint(218, 430, "  Ave:        b/pak");
	Xprint(218, 445, "  Max:        b/pak");
	
	Xprint(390, 325 , "Other");
	Xprint(355, 345, "Slow CPU loss" );
	
	    /*  Draw the quit and reset buttons */
	XDrawRectangle(mydisplay, mywindow, mygc, 10, 450, 40, 40);
	Xprint(17, 472, "Quit");
	XDrawRectangle(mydisplay, mywindow, mygc, 70, 450, 40, 40);
	Xprint(77, 472, "Reset");

}  /* draw the basic window shape */ 

