/*
 * Copyright (C) 1992  Board of Regents of the University of Wisconsin
 * on behalf of the Department of Electrical Engineering and Computer
 * Science, University of Wisconsin-Milwaukee, Milwaukee, WI 53201.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * The programs in this directory were developed by software engineering
 * teams as part of the course "Introduction to Software Engineering"
 * under the supervision of Professor G. Davida.
 *
 * Please send all changes, enhancements, and other comments about this
 * software to
 *
 *     		soft-eng@cs.uwm.edu
 *
 *			or
 *		
 *		Software Engineering Coordinator
 *		Computer Science
 *   		Department of EECS
 *		University of Wisconsin - Milwaukee
 *		Milwaukee, WI  53201
 *		414-229-4677
 */

#include <X11/Xlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <time.h>	  /*  The ultrix Packet filter needs it */
#include <net/if.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>

#include "defs.h"

#ifdef ultrix
#include <net/pfilt.h>
#endif

extern Display *mydisplay;
extern Window mywindow;
extern GC mygc;
extern int filedes;

/*  This structure contains all the data that is sampled and
 *  then printed out later..  I stuck it in a structure so that it
 *  was somewhat organized.
 */

char *othertype = "    ";

struct samp {
int pless65, p65_128, p128_256, p256_512, p512_768, p768_1024, p1024_1280;
int p1280_1518, pgreat1518;
int opless65, op65_128, op128_256, op256_512, op512_768, op768_1024,op1024_1280;
int op1280_1518, opgreat1518;
int other, oother, arp, oarp, ip, oip; 
float netpeak;
float netutil, onetutil;
float bytestotal;
int   bytes;
int   bytespeak;
float packetstotal;
int   packetsmaxsize;
int   packets;
int   packetspeak;
float totaltime;
int   loss;
} sample;


/*  This function is ran at the begging of the program to set
 *  up all the variables to zero
 */
setupvars()
{
	sample.pless65 = sample.p65_128 = sample.p128_256 = sample.p256_512 = 0;
	sample.p512_768 = sample.p768_1024 = sample.p1024_1280=0;
	sample.p1280_1518 = sample.pgreat1518 = 0;
	sample.other = sample.arp = sample.ip = 0;
	sample.bytestotal = sample.packetstotal =0;
	sample.packetsmaxsize= sample.totaltime = 0;
	sample.bytespeak = sample.packetspeak = 0;
	sample.netutil = 0; sample.netpeak = 0;
}

/*  This function is ran after each sample of SAMPLES is read in.
 *  This resets the counts on the type of packets, and to improve
 *  speed, it keeps a count of what the packet distrubution was before.
 */
clearall()
{
	sample.loss = 0;
	sample.packets = 0;
	sample.bytes = 0;

		/* move old values */
	sample.oother = sample.other; sample.oarp = sample.arp;
	sample.oip = sample.ip;
	sample.opless65 = sample.pless65; sample.op65_128 = sample.p65_128; 
	sample.op128_256=sample.p128_256; sample.op256_512 = sample.p256_512; 
        sample.op512_768 = sample.p512_768; 
        sample.op768_1024 = sample.p768_1024; 
        sample.op1024_1280 = sample.p1024_1280;
        sample.op1280_1518 = sample.p1280_1518;
	sample.opgreat1518 = sample.pgreat1518;
	sample.onetutil = sample.netutil;

		/* reset new values */
	sample.other = sample.arp = sample.ip = 0;
	sample.pless65 = sample.p65_128 = sample.p128_256 = sample.p256_512 = 0;
	sample.p512_768 = sample.p768_1024 = sample.p1024_1280=0;
	sample.p1280_1518 = sample.pgreat1518 = 0 ;
}

	/* reset the values when the user presses the button */
void reset_values()
{
	sample.netpeak = sample.bytestotal = sample.bytespeak = 0;
	sample.packetstotal = sample.packetsmaxsize = 0;
	sample.packetspeak = sample.totaltime = 0;
}

/*  This procedure loops for SAMPLES amount of packets, sampling
 *  data, and saving it to the variables in sample
 */
sampledata()
{
   char packetbuffer[BUFFERSIZE];
   int loop, buflen;
   struct enstamp *stamp;
   int currentsize;

   clearall();

   for(loop=0; loop < SAMPLES; loop++) {

/**  Read a packet from the device 
 **/
	     buflen = read(filedes, packetbuffer, BUFFERSIZE);
	     if (buflen == -1) {
	          perror("Error reading from device");
	          exit(1);
 	     }

#ifdef ultrix
	     stamp = (struct enstamp *) packetbuffer;

		/*  This is the true size, not the truncated size
		 *  But it doesn't have the 4byte checksum on it
		 *  anymore 
		 *  Add 4 bytes for the checksum(ultrix only)
		 */
	     currentsize = stamp->ens_count + 4;

		/*  Amount of packets lost from slow cpu  */
	     sample.loss += stamp->ens_dropped;

		/*  Go figure out what type of packet it is.. */
	     figuretype( &(packetbuffer[stamp->ens_stamplen]) );

#else  ultrix
#ifdef sun
	     figuretype( &(packetbuffer[0]) );
#endif sun
		/* If not ultrix set size to be whatever was read in */
	     currentsize = buflen;
#endif ultrix

		/*  figure out the biggest size so far */
	     if (currentsize > sample.packetsmaxsize)
    	           sample.packetsmaxsize = currentsize;

	     sample.bytes += currentsize;

		/* make table of all the sizes -- break into sections */

	     if (currentsize < 65)
   		sample.pless65++;
	     else
  	     if (currentsize <= 128)
   		sample.p65_128++;
	     else
	     if (currentsize <= 256)
   		sample.p128_256++;
	     else
	     if (currentsize <= 512)
   		sample.p256_512++;
	     else
	     if (currentsize <= 768)
   		sample.p512_768++;
	     else
	     if (currentsize <= 1024)
   		sample.p768_1024++;
	     else 
	     if (currentsize <= 1280)
   		sample.p1024_1280++;
	     else 
	     if (currentsize <= 1518)
   		sample.p1280_1518++;
	     else
   		sample.pgreat1518++;

        } /* For loop */

		/*  Compute at the end the new total bytes and packets */
	sample.packets = SAMPLES;
	sample.bytestotal += sample.bytes; 
	sample.packetstotal += sample.packets;

} /* Sample some data */


/*  Figure out what type of packet Only check for simple ones.  */

figuretype(packet)
   u_char *packet;
{
	struct ether_header *header;

	header = (struct ether_header *) packet;

	header->ether_type = (packet[12] << 8) + packet[13];

#ifdef notdef
printf("this %x IP= %x ARP= %x\n", header->ether_type, ETHERTYPE_IP, ETHERTYPE_ARP);
#endif
	if(header->ether_type == ETHERTYPE_IP) {
     	     sample.ip++;
     	     return;
	} 
	if(header->ether_type == ETHERTYPE_ARP) {
         	sample.arp++;
	 	return;
	}
	sample.other++;

	 /*  Don't ask me why this order is reversed!!  It works
	  *  Without reversing it in the sampler program but not
	  *  here!!!!! *&#%! 
	  */
	sprintf(othertype, "%02x%02x", packet[13], packet[12]);

} /* figure type */


/*
 *  Draw all the info on the screen.. Compute all the bytes/sec numbers
 *  and fill everything in 
 */

drawgraphs(time)
float time;
{
	char temp[20];
	int bytes_sec, pak_sec, ave_bytes_sec;
	sample.totaltime += time;

		/********  packet size graph ********/

	bargraph(sample.pless65, sample.opless65, 0);
	bargraph(sample.p65_128, sample.op65_128, 1);
	bargraph(sample.p128_256, sample.op128_256, 2);
	bargraph(sample.p256_512, sample.op256_512, 3);
	bargraph(sample.p512_768, sample.op512_768, 4);
	bargraph(sample.p768_1024, sample.op768_1024, 5);
	bargraph(sample.p1024_1280, sample.op1024_1280, 6);
	bargraph(sample.p1280_1518, sample.op1280_1518, 7);
	bargraph(sample.pgreat1518, sample.opgreat1518, 8);
	
		/****** Packet type graph *******/
	
	Xprint(318, 55, othertype);
	typebargraph(sample.other, sample.oother, 0);
	typebargraph(sample.arp, sample.oarp, 1);
	typebargraph(sample.ip, sample.oip, 2);
	
		/***** Bytes stuff ******/

	sprintf(temp,"%10.0f",sample.bytestotal); Xprint(90, 345, temp);
	
		/*  Bytes per second now  */

	bytes_sec = sample.bytes / time;
	sprintf(temp,"%8d" , bytes_sec); Xprint(103, 365, temp);
	
		/*  Print out Ave bytes/sec  */

	ave_bytes_sec = sample.bytestotal / sample.totaltime;
	sprintf(temp,"%8d", ave_bytes_sec); Xprint(103, 380, temp);
	
		/*  Compute new peak bytes/sec  */

	if(bytes_sec > sample.bytespeak) 
  	sample.bytespeak = bytes_sec;
	sprintf(temp, "%8d", sample.bytespeak); Xprint(103, 395, temp);
	
		/***** Network Utilization..
 		 *****  Note this uses info from above!! ******/

	sample.netutil = bytes_sec / MAXRATE;
	sprintf(temp, "%5.1f", sample.netutil * 100); Xprint(370, 260, temp);
	
	sprintf(temp, "%5.1f", ave_bytes_sec / MAXRATE * 100); Xprint(370, 275, temp);
	if(sample.netutil > sample.netpeak)
  	     sample.netpeak = sample.netutil;
	
	sprintf(temp, "%5.1f", sample.netpeak * 100); Xprint(370, 290, temp);
	
		/*****  Load over time *****/

	loadovertime(sample.netutil);
	
		/*****  Nifty bar network load line ****/

	loadbargraph(sample.netutil, sample.onetutil);
	
		/*****  Packet stuff   *****/

	sprintf(temp, "%6.0f", sample.packetstotal); Xprint(260, 345, temp);
	
		/*  pak per sec stuff */
	pak_sec = SAMPLES / time;
	sprintf(temp, "%6d", pak_sec); Xprint(260, 365, temp);
	sprintf(temp, "%6.0f", sample.packetstotal/sample.totaltime);
	Xprint(260, 380, temp);
	if(pak_sec > sample.packetspeak)
   	     sample.packetspeak = pak_sec;
	sprintf(temp, "%6d", sample.packetspeak); Xprint(260, 395, temp);
	
	sprintf(temp, "%6d",  sample.bytes / SAMPLES); Xprint(260, 415, temp);
	sprintf(temp, "%6.0f",sample.bytestotal / sample.packetstotal);
	Xprint(260, 430, temp);
	sprintf(temp, "%6d", sample.packetsmaxsize); Xprint(260, 445, temp);
	
		/*******  Other box ***********/

	sprintf(temp, "%4d", sample.loss); Xprint(437, 345, temp);

} /* Draw Graphs */

